"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("@tarkvaramehed/errors");
const pg_1 = require("pg");
const postgres_migrations_1 = require("postgres-migrations");
const slonik_1 = require("slonik");
const path_1 = require("path");
const moment_1 = __importDefault(require("moment"));
const perf_hooks_1 = require("perf_hooks");
function initialize(conf) {
    var _a, _b, _c;
    return __awaiter(this, void 0, void 0, function* () {
        pg_1.types.setTypeParser(pg_1.types.builtins.DATE, val => val === null ? null : moment_1.default(val).format('YYYY-MM-DD'));
        const { database, host, port, user, password, migrations, logger = exports.defaultLogger } = conf;
        logger.info('Running database initialization');
        logger.info({
            database,
            host,
            port,
            user,
            password,
            migrations
        });
        let client = new pg_1.Client({ host, port, user, password });
        let res;
        logger.info('Connecting to client');
        yield client.connect();
        try {
            logger.info(`Checking if database ${database} exist`);
            res = yield client.query('SELECT EXISTS ( SELECT datname FROM pg_catalog.pg_database WHERE LOWER ( datname ) = LOWER( $1 ) ) AS "dbExists"', [database]);
        }
        catch (error) {
            logger.error('Failed to connect to database', error);
            return void (yield client.end());
        }
        if ((_b = (_a = res === null || res === void 0 ? void 0 : res.rows) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b['dbExists']) {
            logger.info(`database ${database} already exists, skipping initialization`);
        }
        else {
            try {
                logger.info(`Attempting to create database ${database}`);
                yield postgres_migrations_1.createDb(database, { client });
                logger.info(`Database ${database} created successfully`);
            }
            catch (error) {
                logger.error('Failed to initialize database', error);
            }
        }
        if (migrations) {
            yield client.end();
            client = new pg_1.Client({ host, port, user, password, database });
            yield client.connect();
            try {
                logger.info(`Attempting to run migrations on database ${database}`);
                yield postgres_migrations_1.migrate({ client }, path_1.resolve(migrations));
                logger.info(`Migrations ran successfully`);
            }
            catch (error) {
                logger.error('Failed to do migrations', error);
            }
        }
        yield client.end();
        (_c = conf.onReady) === null || _c === void 0 ? void 0 : _c.call(conf);
        return;
    });
}
exports.initialize = initialize;
function readCursor(rows, cursor) {
    return new Promise((resolve, reject) => cursor.read(rows, (err, res) => {
        if (err) {
            return reject(new errors_1.DatabaseError(err.message));
        }
        const count = res.length;
        if (count === 0) {
            cursor.close();
            return resolve({
                result: {
                    rows: [],
                    rowCount: 0
                }
            });
        }
        return resolve({
            next: (r) => readCursor(r || rows, cursor),
            result: {
                rowCount: count,
                rows: res
            },
            close: () => cursor.close()
        });
    }));
}
exports.readCursor = readCursor;
function handleSlonikError(error) {
    if (error instanceof slonik_1.SlonikError) {
        if (error instanceof slonik_1.BackendTerminatedError) {
            return new errors_1.DatabaseBackendTerminatedError(error);
        }
        if (error instanceof slonik_1.CheckIntegrityConstraintViolationError) {
            return new errors_1.DatabaseCheckConstraintError(error);
        }
        if (error instanceof slonik_1.ConnectionError) {
            return new errors_1.DatabaseConnectionError(error);
        }
        if (error instanceof slonik_1.DataIntegrityError) {
            return new errors_1.DatabaseDataIntegrityError(error);
        }
        if (error instanceof slonik_1.ForeignKeyIntegrityConstraintViolationError) {
            return new errors_1.DatabaseForeignKeyConstraintError(error);
        }
        if (error instanceof slonik_1.NotFoundError) {
            return new errors_1.DatabaseNotFoundError(error);
        }
        if (error instanceof slonik_1.NotNullIntegrityConstraintViolationError) {
            return new errors_1.DatabaseNotNullConstraintError(error);
        }
        if (error instanceof slonik_1.UniqueIntegrityConstraintViolationError) {
            return new errors_1.DatabaseUniqueConstraintError(error);
        }
    }
    return new errors_1.DatabaseError(error);
}
function handleQuery(queryMethod, query, logger) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            try {
                logger.info(`Querying from database`, {
                    query: query.sql
                });
                const t0 = perf_hooks_1.performance.now();
                resolve(yield queryMethod(query));
                const t1 = perf_hooks_1.performance.now();
                logger.info(`Query finished successfully`, {
                    time: `${(t1 - t0).toFixed(4)} ms`,
                    query: query.sql
                });
            }
            catch (error) {
                logger.error(`Query resulted in an error`, Object.assign(Object.assign({}, error), { query: query.sql }));
                reject(handleSlonikError(error));
            }
        }));
    });
}
exports.handleQuery = handleQuery;
exports.defaultLogger = {
    info: (...args) => console.info(...args),
    error: (...args) => console.error(...args)
};
//# sourceMappingURL=util.js.map